unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,  typinfo,   ComObj, ActiveX,
  Series,
  Chart,
  Jpeg,
  TeEngine, ExtCtrls;

type
  TTester = class(TThread)
    n: Integer;
    procedure Execute(); override;
  end;

  TForm1 = class(TForm)
    Button1: TButton;
    Label1: TLabel;
    Timer1: TTimer;
    procedure Button1Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    FTester: TTester;
  public
    { Public declarations }
    procedure TesterFinished();
    procedure TesterFailed();
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

var AppRoot: String;

Function GetChartBMP(AChart:TCustomChart):TBitmap;
var tmpBitmap:TBitmap;
begin
  tmpBitmap:=TBitmap.Create;
  tmpBitmap.Width :=AChart.Width;
  tmpBitmap.Height:=AChart.Height;

  AChart.Draw(tmpBitmap.Canvas,Rect(0,0,tmpBitmap.Width,tmpBitmap.Height));

  Result := tmpBitmap;
end;

procedure CreateChartFile(ATitle: String; const AFileName: WideString);
var
  Chart: TChart;
  Series : TBarSeries;
  i: Integer;
  Rows: Integer;
  ChartWidth, ChartHeight: Integer;
begin
  Rows := 20;
  ChartWidth := 800;
  ChartHeight := 400;

  Chart := TChart.Create(nil);
  try
    Series := TBarSeries.Create(nil);
    TBarSeries(Series).BarStyle := bsRectangle;
    Chart.LeftAxis.Title.Caption := 'Left Axis';

    Chart.AddSeries(Series);

    Chart.Legend.TextStyle := ltsPlain;
    Chart.Legend.Alignment := laRight;

    Chart.Height := ChartHeight;
    Chart.Width := ChartWidth;
    Chart.Color := clWhite;
    Chart.BevelInner := bvNone;
    Chart.BevelOuter := bvNone;
    Chart.BottomWall.Color := clWhite;
    Chart.LeftWall.Color := clWhite;

    Chart.Title.Text.Text := ATitle;
    with Chart.Title.Font do
    begin
     Name := 'Arial';
     Size := 12;
     Style := [fsBold];
    end;

    // series

    Series.Marks.Visible := False;

    for i := 0 to Rows - 1 do
     Series.Add(i, 'label'+IntToStr(i));

    Series.SeriesColor := clBlack;
    Series.ColorEachPoint := true;

    //Chart.SaveToBitmapFile(AFileName);
    //with Chart.TeeCreateBitmap(clWhite, Rect(0, 0, ChartWidth, ChartHeight)) do
    with GetChartBMP(Chart) do
     try
       SaveToFile(AFileName);
     finally
       Free();
     end;
  finally
    Chart.Free;
  end;
end;

function equalfiles(const fname1, fname2: string): Boolean;
var
  m1, m2: TMemoryStream;
begin
  m1 := NIL;
  m2 := NIL;
  try
    m1 := TMemoryStream.Create;
    m2 := TMemoryStream.Create;

    m1.LoadFromFile(fname1);
    m2.LoadFromFile(fname2);

    result := (m1.Size = m2.Size);

    if (result)
     then result := CompareMem(m1.Memory, m2.Memory, m1.Size)
  finally
    m1.Free;
    m2.Free;
  end;
end;

procedure TTester.Execute();
var error: boolean;
begin
  // base chart
  DeleteFile(AppRoot+'chart.bmp');
  CreateChartFile('Test chart', AppRoot+'chart.bmp');

  n := 0;
  error := false;

  try

    while (not terminated) do
     begin
       // delete previous
       if (FileExists(AppRoot+'chartX.bmp'))
        then ASSERT(DeleteFile(AppRoot+'chartX.bmp'));

       // create next
       CreateChartFile('Test chart', AppRoot+'chartX.bmp');

       // compare
       if (not equalfiles(AppRoot+'chart.bmp', AppRoot+'chartX.bmp'))
        then
         begin
           Synchronize(Form1.TesterFailed);
           error := true;
           break;
         end;

       inc(n);
     end;

  finally

    if (not error)
     then Synchronize(Form1.TesterFinished);

  end;
end;

/////////// ------ FORM -------- ////////////////////

procedure TForm1.Button1Click(Sender: TObject);
begin
  if (FTester <> NIL)
   then
    begin
      Timer1.Enabled := false;
      FTester.Terminate();
      FTester.WaitFor();
      FreeAndNil(FTester);
      Button1.Caption := 'Start';
    end
   else
    begin
     Button1.Caption := 'Stop';
     FTester := TTester.Create(true);
     FTester.Resume();

     Timer1.Enabled := true;
     Label1.Caption := 'Testing..';
    end;
end;

procedure TForm1.TesterFailed;
begin
  Timer1.Enabled := false;
  Label1.Caption := 'Test failed: The files are different!';
end;

procedure TForm1.TesterFinished;
begin
  Label1.Caption := 'Done.';
end;

// Use messages
const A = '-\|/';
var ll:integer=0;
procedure TForm1.Timer1Timer(Sender: TObject);
begin
  Caption := IntToStr(FTester.n)+' done, working '+A[ll+1];
  ll := (ll + 1) mod 4;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (FTester <> NIL)
   then
    begin
      FTester.Terminate;
      FTester.WaitFor();
    end;
end;

initialization
  AppRoot := IncludeTrailingPathDelimiter(ExtractFilePath(ParamStr(0)));
end.
